@@ -100,9 +100,9 @@ namespace SExpEngine
         {
             bool in_string = false;
             bool in_comment = false;
-            
-            while(true) {
-                int ich = reader.Read();
+
+            int ich = reader.Read();
+            while(ich != -1) {
                 char ch = (char)ich;
                 
                 if(ich < 0) {
@@ -170,8 +170,9 @@ namespace SExpEngine
                         } else {
                             current_token.Append(ch);
                         }
-                        break;    
+                        break;
                 }
+                ich = reader.Read();
             }
             
             TokenPush(false);
