@@ -9,6 +9,7 @@
 //   tag.cpp from TagLib
 //
 // Copyright (C) 2005-2007 Brian Nickel
+// Copyright (C) 2003 Scott Wheeler
 // 
 // This library is free software; you can redistribute it and/or modify
 // it  under the terms of the GNU Lesser General Public License version
@@ -302,6 +303,9 @@ namespace TagLib {
 		///    if no value is present.
 		/// </value>
 		/// <remarks>
+		///    <para>Years greater than 9999 cannot be stored by most
+		///    tagging formats and will be cleared if a higher value is
+		///    set.</para>
 		///    <para>Some tagging formats store higher precision dates
 		///    which will be truncated when this property is set. Format
 		///    specific implementations are necessary access the higher
@@ -630,12 +634,12 @@ namespace TagLib {
 		}
 		
 		/// <summary>
-		///    Gets a comma separated string containing the values in
-		///    <see cref="AlbumArtists" />.
+		///    Gets a semicolon separated string containing the values
+		///    in <see cref="AlbumArtists" />.
 		/// </summary>
 		/// <value>
-		///    A comma separated <see cref="string" /> object containing
-		///    the values in <see cref="AlbumArtists" />.
+		///    A semicolon separated <see cref="string" /> object
+		///    containing the values in <see cref="AlbumArtists" />.
 		/// </value>
 		/// <remarks>
 		///    This property is provided for convenience. Use <see
@@ -646,12 +650,12 @@ namespace TagLib {
 		}
 		
 		/// <summary>
-		///    Gets a comma separated string containing the values in
-		///    <see cref="Performers" />.
+		///    Gets a semicolon separated string containing the values
+		///    in <see cref="Performers" />.
 		/// </summary>
 		/// <value>
-		///    A comma separated <see cref="string" /> object containing
-		///    the values in <see cref="Performers" />.
+		///    A semicolon separated <see cref="string" /> object
+		///    containing the values in <see cref="Performers" />.
 		/// </value>
 		/// <remarks>
 		///    This property is provided for convenience. Use <see
@@ -662,12 +666,12 @@ namespace TagLib {
 		}
 		
 		/// <summary>
-		///    Gets a comma separated string containing the values in
-		///    <see cref="Composers" />.
+		///    Gets a semicolon separated string containing the values
+		///    in <see cref="Composers" />.
 		/// </summary>
 		/// <value>
-		///    A comma separated <see cref="string" /> object containing
-		///    the values in <see cref="Composers" />.
+		///    A semicolon separated <see cref="string" /> object
+		///    containing the values in <see cref="Composers" />.
 		/// </value>
 		/// <remarks>
 		///    This property is provided for convenience. Use <see
@@ -678,12 +682,12 @@ namespace TagLib {
 		}
 		
 		/// <summary>
-		///    Gets a comma separated string containing the values in
-		///    <see cref="Genres" />.
+		///    Gets a semicolon separated string containing the values
+		///    in <see cref="Genres" />.
 		/// </summary>
 		/// <value>
-		///    A comma separated <see cref="string" /> object containing
-		///    the values in <see cref="Genres" />.
+		///    A semicolon separated <see cref="string" /> object
+		///    containing the values in <see cref="Genres" />.
 		/// </value>
 		/// <remarks>
 		///    This property is provided for convenience. Use <see
@@ -707,23 +711,26 @@ namespace TagLib {
 		private static string FirstInGroup(string [] group)
 		{
 			return group == null || group.Length == 0 ?
-				null : group[0];
+				null : group [0];
 		}
 		
 		/// <summary>
-		///    Joins a array of strings into a single, comma separated,
-		///    string.
+		///    Joins a array of strings into a single, semicolon
+		///    separated, string.
 		/// </summary>
 		/// <param name="group">
 		///    A <see cref="string[]" /> containing values to combine.
 		/// </param>
 		/// <returns>
-		///    A comma separated <see cref="string" /> object containing
-		///    the values from <paramref name="group" />.
+		///    A semicolon separated <see cref="string" /> object
+		///    containing the values from <paramref name="group" />.
 		/// </returns>
-		private static string JoinGroup(string [] group)
+		private static string JoinGroup (string [] group)
 		{
-			return new StringCollection(group).ToString(", ");
+			if (group == null)
+				return null;
+			
+			return string.Join ("; ", group);
 		}
 
 		/// <summary>
@@ -797,6 +804,7 @@ namespace TagLib {
 		///    <paramref name="source" /> or <paramref name="target" />
 		///    is <see langword="null" />.
 		/// </exception>
+		[Obsolete("Use Tag.CopyTo(Tag,bool)")]
 		public static void Duplicate (Tag source, Tag target,
 		                              bool overwrite)
 		{
@@ -806,53 +814,85 @@ namespace TagLib {
 			if (target == null)
 				throw new ArgumentNullException ("target");
 			
+			source.CopyTo (target, overwrite);
+		}
+		
+		/// <summary>
+		///    Copies the values from the current instance to another
+		///    <see cref="TagLib.Tag" />, optionally overwriting
+		///    existing values.
+		/// </summary>
+		/// <param name="target">
+		///    A <see cref="Tag" /> object containing the target tag to
+		///    copy values to.
+		/// </param>
+		/// <param name="overwrite">
+		///    A <see cref="bool" /> specifying whether or not to copy
+		///    values over existing one.
+		/// </param>
+		/// <remarks>
+		///    <para>This method only copies the most basic values when
+		///    copying between different tag formats, however, if
+		///    <paramref name="target" /> is of the same type as the
+		///    current instance, more advanced copying may be done.
+		///    For example, <see cref="TagLib.Id3v2.Tag" /> will copy
+		///    all of its frames to another tag.</para>
+		/// </remarks>
+		/// <exception cref="ArgumentNullException">
+		///    <paramref name="target" /> is <see langword="null" />.
+		/// </exception>
+		public virtual void CopyTo (Tag target, bool overwrite)
+		{
+			if (target == null)
+				throw new ArgumentNullException ("target");
+			
 			if (overwrite || IsNullOrLikeEmpty (target.Title))
-				target.Title = source.Title;
+				target.Title = Title;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.AlbumArtists))
-				target.AlbumArtists = source.AlbumArtists;
+				target.AlbumArtists = AlbumArtists;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Performers))
-				target.Performers = source.Performers;
+				target.Performers = Performers;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Composers))
-				target.Composers = source.Composers;
+				target.Composers = Composers;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Album))
-				target.Album = source.Album;
+				target.Album = Album;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Comment))
-				target.Comment = source.Comment;
+				target.Comment = Comment;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Genres))
-				target.Genres = source.Genres;
+				target.Genres = Genres;
 			
 			if (overwrite || target.Year == 0)
-				target.Year = source.Year;
+				target.Year = Year;
 			
 			if (overwrite || target.Track == 0)
-				target.Track = source.Track;
+				target.Track = Track;
 			
 			if (overwrite || target.TrackCount == 0)
-				target.TrackCount = source.TrackCount;
+				target.TrackCount = TrackCount;
 			
 			if (overwrite || target.Disc == 0)
-				target.Disc = source.Disc;
+				target.Disc = Disc;
 			
 			if (overwrite || target.DiscCount == 0)
-				target.DiscCount = source.DiscCount;
+				target.DiscCount = DiscCount;
 			
 			if (overwrite || target.BeatsPerMinute == 0)
-				target.BeatsPerMinute = source.BeatsPerMinute;
+				target.BeatsPerMinute = BeatsPerMinute;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Grouping))
-				target.Grouping = source.Grouping;
+				target.Grouping = Grouping;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Conductor))
-				target.Conductor = source.Conductor;
+				target.Conductor = Conductor;
 			
 			if (overwrite || IsNullOrLikeEmpty (target.Copyright))
-				target.Conductor = source.Copyright;
+				target.Conductor = Copyright;
 		}
 		
 		/// <summary>
