@@ -80,19 +80,21 @@ namespace Banshee.Dap
             } else {
                 table.AddLabel(Catalog.GetString("Device name"), device.Name);
             }
-                        
-            if(!device.IsReadOnly && device.CanSetOwner) {
-                ownerEntry = table.AddEntry(Catalog.GetString("Owner name"), device.Owner);
-                ownerEntry.Changed += OnEntryChanged;
-            } else {
-                table.AddLabel(Catalog.GetString("Owner name"), device.Owner);
+            
+            if(device.ShowOwner) {
+                if(!device.IsReadOnly && device.CanSetOwner) {
+                    ownerEntry = table.AddEntry(Catalog.GetString("Owner name"), device.Owner);
+                    ownerEntry.Changed += OnEntryChanged;
+                } else {
+                    table.AddLabel(Catalog.GetString("Owner name"), device.Owner);
+                }
             }
-    
+            
             if(!device.IsReadOnly) {   
                 try {
                     VBox profile_description_box = new VBox();
                     ProfileComboBoxConfigurable profile_box = new ProfileComboBoxConfigurable(Globals.AudioProfileManager, 
-                        device.ID, profile_description_box);
+                        device.Uuid, profile_description_box);
                     profile_box.Combo.MimeTypeFilter = device.SupportedPlaybackMimeTypes;
                     table.AddWidget(Catalog.GetString("Encode to"), profile_box);
                     table.AddWidget(null, profile_description_box);
