@@ -107,7 +107,9 @@ namespace Banshee.Gui
             focus_column.SetCellDataFunc(renderer, new TreeCellDataFunc(SourceCellDataFunc));
             AppendColumn(focus_column);
             
-            store = new TreeStore(typeof(Source));
+            store = new TreeStore(typeof(Source), typeof(int));
+            store.SetSortColumnId (1, SortType.Ascending);
+            store.ChangeSortColumn ();
             Model = store;
             HeadersVisible = false;
             
@@ -119,7 +121,7 @@ namespace Banshee.Gui
             RefreshList();
 
             SourceManager.SourceAdded += delegate(SourceAddedArgs args) {
-                AddSource(args.Source, args.Position);
+                AddSource(args.Source);
             };
             
             SourceManager.SourceRemoved += delegate(SourceEventArgs args) {
@@ -131,6 +133,8 @@ namespace Banshee.Gui
             };
             
             SourceManager.SourceUpdated += delegate(SourceEventArgs args) {
+                TreeIter iter = FindSource (args.Source);
+                store.SetValue (iter, 1, args.Source.Order);
                 QueueDraw();
             };
         }
@@ -168,34 +172,36 @@ namespace Banshee.Gui
         
         private void AddSource(Source source)
         {
-            AddSource(source, -1);
+            AddSource(source, TreeIter.Zero);
         }
 
-        private void AddSource(Source source, int position)
+        private void AddSource(Source source, TreeIter parent)
         {
-            AddSource(source, position, TreeIter.Zero);
-        }
-        
-        private void AddSource(Source source, int position, TreeIter parent)
-        {
-            if(!FindSource(source).Equals(TreeIter.Zero)) {
+            // Don't add duplicates
+            if(!FindSource(source).Equals(TreeIter.Zero))
                 return;
-            }
+
+            // Don't add a child source before its parent
+            if(parent.Equals(TreeIter.Zero) && source is ChildSource)
+                return;
+
+            int position = source.Order;
             
             TreeIter iter = parent.Equals(TreeIter.Zero)
                 ? store.InsertNode(position) 
-                : store.AppendNode(parent);
+                : store.InsertNode(parent, position);
             
             store.SetValue(iter, 0, source);
+            store.SetValue(iter, 1, source.Order);
 
             lock(source.Children) {
                 foreach(ChildSource s in source.Children) {
-                    AddSource(s, position, iter);
+                    AddSource(s, iter);
                 }
             }
 
             source.ChildSourceAdded += delegate(SourceEventArgs e) {
-                AddSource(e.Source, position, iter);
+                AddSource(e.Source, iter);
             };
 
             source.ChildSourceRemoved += delegate(SourceEventArgs e) {
@@ -304,7 +310,6 @@ namespace Banshee.Gui
             if(iter.Equals(TreeIter.Zero)) {
                 return;
             }
-            
             renderer.Editable = true;
             SetCursor(store.GetPath(iter), focus_column, true);
             renderer.Editable = false;
@@ -319,9 +324,13 @@ namespace Banshee.Gui
             }
 
             Source source = GetSource(path);
-
-            if(evnt.Button == 1 && evnt.Type == EventType.TwoButtonPress) {
+            if(evnt.Button == 1) {
                 if(!source.CanActivate) {
+                    if(!source.Expanded) {
+                        ExpandRow(path, false);
+                    } else {
+                        CollapseRow(path);
+                    }
                     return false;
                 }
                 
@@ -329,9 +338,10 @@ namespace Banshee.Gui
                     SourceManager.SetActiveSource(source);
                 }
                 
-                OnSourceDoubleClicked();
-
-                return false;
+                if(evnt.Type == EventType.TwoButtonPress) {
+                    OnSourceDoubleClicked();
+                }
+                
             } else if(evnt.Button == 3) {
                 HighlightPath(path);
 
@@ -436,6 +446,7 @@ namespace Banshee.Gui
                 TreeIter library = FindSource(LibrarySource.Instance);
                 newPlaylistIter = store.AppendNode(library);
                 store.SetValue(newPlaylistIter, 0, newPlaylistSource);
+                store.SetValue(newPlaylistIter, 1, 999);
                 newPlaylistVisible = true;
 
                 UpdateView();
@@ -529,6 +540,12 @@ namespace Banshee.Gui
                     LogCore.Instance.PushError(Catalog.GetString("Could not import tracks"), e.Message);
                 }
             }
+            
+            if(newPlaylistVisible) {
+                store.Remove(ref newPlaylistIter);
+                newPlaylistVisible = false;
+                UpdateView();
+            }
         
             Gtk.Drag.Finish(context, true, false, time);
         }
@@ -699,6 +716,10 @@ namespace Banshee.Gui
             titleLayout.FontDescription = fd;
             countLayout.FontDescription = fd;
             
+            titleLayout.SetMarkup("...");
+            titleLayout.GetPixelSize(out titleLayoutWidth, out titleLayoutHeight);
+            int ellipsisSize = titleLayoutWidth;
+            
             string titleText = source.Name;
             titleLayout.SetMarkup(GLib.Markup.EscapeText(titleText));
             countLayout.SetMarkup("<span size=\"small\">(" + source.Count + ")</span>");
@@ -708,20 +729,19 @@ namespace Banshee.Gui
             
             maxTitleLayoutWidth = cell_area.Width - icon.Width - countLayoutWidth - 10;
             
-            while(true) {
-                titleLayout.GetPixelSize(out titleLayoutWidth, out titleLayoutHeight);
-                if(titleLayoutWidth <= maxTitleLayoutWidth) {
-                    break;
-                }
-                
-                // FIXME: Gross
-                try {
-                    titleText = titleText.Substring(0, titleText.Length - 1);
-                    titleLayout.SetMarkup(GLib.Markup.EscapeText(titleText).Trim() + "...");
-                } catch(Exception) {
-                    titleLayout.SetMarkup(source.Name);
-                    hideCounts = true;
-                    break;
+            if(titleLayoutWidth > maxTitleLayoutWidth) {
+                float ratio = (float)(maxTitleLayoutWidth - ellipsisSize) / (float)titleLayoutWidth;
+                int characters = (int)(ratio * (float)titleText.Length);
+                while(titleLayoutWidth > maxTitleLayoutWidth) {
+                    if(characters > 0) {
+                        titleLayout.SetMarkup(GLib.Markup.EscapeText(
+                            titleText.Substring(0, characters--)).Trim() + "...");
+                        titleLayout.GetPixelSize(out titleLayoutWidth, out titleLayoutHeight);
+                    } else {
+                        hideCounts = true;
+                        titleLayout.SetMarkup(GLib.Markup.EscapeText(titleText.Trim()));
+                        break;
+                    }
                 }
             }
             
