@@ -205,7 +205,7 @@ namespace Banshee.Dap.MassStorage
             if(PlaybackFormats.Length > 0) {
                 InstallProperty(String.Format(
                     Catalog.GetPluralString("Audio Format", "Audio Formats", PlaybackFormats.Length), PlaybackFormats.Length),
-                    System.String.Join("\n", PlaybackFormats)
+                    System.String.Join(", ", PlaybackFormats)
                 );
             }
 
@@ -254,10 +254,17 @@ namespace Banshee.Dap.MassStorage
             if(mounted)
                 ReloadDatabase();
         }
+        
+        protected override void Reload ()
+        {
+            ReloadDatabase();
+        }
  
         private void ReloadDatabase()
         {
-            ClearTracks(false);
+            lock(Source.TracksMutex) {
+                ClearTracks(false);
+            }
 
             ImportManager importer = new ImportManager();
 
@@ -325,19 +332,8 @@ namespace Banshee.Dap.MassStorage
 
         public override void Eject()
         {
-            // If we're playing a track on the device, stop playing it before trying to eject
-            if(PlayerEngineCore.CurrentTrack is MassStorageTrackInfo) {
-                LogCore.Instance.PushInformation(
-                    Catalog.GetString("Song Playing on Device"),
-                    Catalog.GetString("Before you can eject your device, you need to start playing a song that is not on it.  This is a known bug."),
-                    true
-                );
-
-                //PlayerEngineCore.StateChanged += HandleStopped;
-                //PlayerEngineCore.Close();
-            } else {
-                Unmount();
-            }
+            UnmapPlayback(typeof(MassStorageTrackInfo));
+            Unmount();
         }
 
         private void HandleStopped(object o, Banshee.MediaEngine.PlayerEngineStateArgs args)
@@ -390,7 +386,9 @@ namespace Banshee.Dap.MassStorage
             if(track == null || IsReadOnly && !tracks.Contains(track))
                 return;
 
-            tracks.Add(track);
+            lock(Source.TracksMutex) {
+                tracks.Add(track);
+            }
             OnTrackAdded(track);
 
             if(!(track is MassStorageTrackInfo)) {
@@ -420,11 +418,15 @@ namespace Banshee.Dap.MassStorage
                     TrackInfo new_track = new MassStorageTrackInfo(new SafeUri(new_path));
 
                     // Add the new MassStorageTrackInfo and remove the old TrackInfo from the treeview
-                    tracks.Add(new_track);
+                    lock(Source.TracksMutex) {
+                        tracks.Add(new_track);
+                    }
                     OnTrackAdded(new_track);
                 }
 
-                tracks.Remove(track);
+                lock(Source.TracksMutex) {
+                    tracks.Remove(track);
+                }
             } catch (Exception e) {
                 LogCore.Instance.PushInformation("Unable to copy file", new_path + e.ToString(), false);
             }
@@ -473,11 +475,23 @@ namespace Banshee.Dap.MassStorage
             } catch {}
         }
 
-        public override Gdk.Pixbuf GetIcon(int size)
+        private Dictionary<int, Gdk.Pixbuf> icon_cache = new Dictionary<int, Gdk.Pixbuf>(2);
+        const string icon_prefix = "multimedia-player-";
+        public override Gdk.Pixbuf GetIcon (int size)
         {
-            string prefix = "multimedia-player";
-            Gdk.Pixbuf icon = IconThemeUtils.LoadIcon(prefix + ((IconId == null) ? "" : "-" + IconId), size);
-            return icon == null ? base.GetIcon(size) : icon;
+            if (icon_cache.ContainsKey(size))
+                return icon_cache[size];
+
+            Gdk.Pixbuf icon = null;
+            string icon_id = IconId;
+
+            if (icon_id != null && IconThemeUtils.HasIcon (icon_prefix + icon_id))
+                icon = IconThemeUtils.LoadIcon (icon_prefix + icon_id, size);
+            else
+                icon = base.GetIcon (size);
+
+            icon_cache.Add(size, icon);
+            return icon;
         }
 
         private string GetTrackPath(TrackInfo track)
@@ -546,22 +560,102 @@ namespace Banshee.Dap.MassStorage
             set { remover = value; }
         }
 
+        private string icon_id = null;
         public virtual string IconId {
             get {
-                return null;
+                if (icon_id == null) {
+                    string vendor = player_device["info.vendor"];
+                    string product = player_device["info.product"];
+                    if (vendor != null && vendor != String.Empty && product != null && product != String.Empty)
+                        icon_id = (vendor.Trim () + "-" + product.Trim ()).Replace (' ', '-').ToLower ();
+                }
+
+                return icon_id;
             }
         }
+        
+        private string GetDisplayStringProperty(Hal.Device device, string property)
+        {
+            if(device.PropertyExists(property)) {
+                string value = device[property];
+                if(String.IsNullOrEmpty(value)) {
+                    return null;
+                }
+                
+                return value;
+            }
+            
+            return null;
+        }
+ 
+        private Stack<Hal.Device> CollectUsbDeviceStack(Hal.Device device)
+        {
+            Stack<Hal.Device> device_stack = new Stack<Hal.Device>();
+            int usb_vendor_id = -1;
+            int usb_product_id = -1;
+
+            Hal.Device tmp_device = device;
+
+            while(tmp_device != null) {
+                // Skip the SCSI parents of the player volume if they are in the tree
+                if((tmp_device.PropertyExists("info.bus") && tmp_device["info.bus"] == "scsi") ||
+                    (tmp_device.PropertyExists("info.category") && tmp_device["info.category"] == "scsi_host")) {
+                    device_stack.Push(tmp_device);
+                    tmp_device = tmp_device.Parent;
+                    continue;
+                }
+
+                bool have_usb_ids = false;
+                int _usb_vendor_id = -1;
+                int _usb_product_id = -1;
+
+                // Figure out the IDs if they exist
+                if(tmp_device.PropertyExists("usb.vendor_id") && tmp_device.PropertyExists("usb.product_id")) {
+                    _usb_vendor_id = tmp_device.GetPropertyInteger("usb.vendor_id");
+                    _usb_product_id = tmp_device.GetPropertyInteger("usb.product_id");
+                    have_usb_ids = true;
+                } else if(tmp_device.PropertyExists("usb_device.vendor_id") && tmp_device.PropertyExists("usb_device.product_id")) {
+                    _usb_vendor_id = tmp_device.GetPropertyInteger("usb_device.vendor_id");
+                    _usb_product_id = tmp_device.GetPropertyInteger("usb_device.product_id");
+                    have_usb_ids = true;
+                }
+
+                if(have_usb_ids) {
+                    if(usb_vendor_id == -1 && usb_product_id == -1) {
+                        // We found the first raw USB device, remember it
+                        usb_vendor_id = _usb_vendor_id;
+                        usb_product_id = _usb_product_id;
+                    } else if(usb_vendor_id != _usb_vendor_id || usb_product_id != _usb_product_id) {
+                        // We are no longer looking at the device we care about (could now be at a hub or something)
+                        break;
+                    }
+                } else if(usb_vendor_id != -1 || usb_product_id != -1) {
+                    // We are no longer even looking at USB devices
+                    break;
+                }
+
+                device_stack.Push(tmp_device);
+                tmp_device = tmp_device.Parent;
+            }
+            
+            return device_stack;
+        }
  
         private string name = null;
         public override string Name {
             get {
                 if(name == null) {
-                    if(player_device.PropertyExists("info.product")) {
-                        name = player_device["info.product"];
-                    } else if(volume_device.PropertyExists("volume.label") && 
-                        volume_device["volume.label"].Length > 0) {
-                        name = volume_device["volume.label"];
-                    } else {
+                    Stack<Hal.Device> usb_devices = CollectUsbDeviceStack(player_device);
+                    
+                    while(usb_devices.Count > 0) {
+                        name = GetDisplayStringProperty(usb_devices.Pop(), "info.product");
+                        if(name != null) {
+                            return name;
+                        }
+                    }
+                
+                    name = GetDisplayStringProperty(volume_device, "volume.label");
+                    if(name == null) {
                         name = GenericName;
                     }
                 }
