@@ -288,11 +288,22 @@ namespace Banshee.Plugins.NotificationAreaIcon
                 Notification nf = new Notification(Catalog.GetString("Now Playing"), message, image, event_box);
                 nf.Urgency = Urgency.Low;
                 nf.Timeout = 4500;
+                nf.AddHint("desktop-entry", "banshee.desktop");
+                if(Globals.ActionManager["NextAction"].Sensitive) {
+                    nf.AddAction("skip-song", Catalog.GetString("Play next song"), OnSongSkipped);
+                }
                 nf.Show();
             } catch(Exception e) {
                 LogCore.Instance.PushError(Catalog.GetString("Cannot show notification"), e.Message, false);
             }
         }
+        
+        private void OnSongSkipped(object o, ActionArgs args)
+        {
+            if(args.Action == "skip-song") {
+                Globals.ActionManager["NextAction"].Activate();
+            }
+        }
 
         [GLib.ConnectBefore]
         private void OnKeyPressEvent(object o, KeyPressEventArgs args)
@@ -509,7 +520,10 @@ namespace Banshee.Plugins.NotificationAreaIcon
             popup.Artist = PlayerEngineCore.CurrentTrack.DisplayArtist;
             popup.Album = PlayerEngineCore.CurrentTrack.Album;
             popup.TrackTitle = PlayerEngineCore.CurrentTrack.DisplayTitle;
-            popup.CoverArtFileName = PlayerEngineCore.CurrentTrack.CoverArtFileName;
+            try {
+                popup.CoverArtFileName = PlayerEngineCore.CurrentTrack.CoverArtFileName;
+            } catch {
+            }
             popup.QueueDraw();
             if (!popup.Visible) {
                 PositionPopup();
